from .enums import DeviceType


def parse_device_type(type: int):
    if type == 6 or type == 2:
        return DeviceType.LIGHT
    if type == 0:
        return DeviceType.FAN
    else:
        return DeviceType.SWITCH



import aiohttp
import asyncio
import json

from .utils import *
from . import constants
from .user import User
from .device import Device
from .home import Home
from .broker import AlisteBroker


class AlisteHub:
    home: Home

    def __init__(self):
        self.http = aiohttp.ClientSession()
        self.background_tasks = set()
        self.broker = AlisteBroker()

    async def __aenter__(self):
        return self

    async def __aexit__(self, *excinfo):
        await self.http.close()
        await self.broker.disconnect()

    async def init(self, username: str, password: str):
        await self._authenticate(username, password)
        await self.get_home_details()
        await self._init_broker()

    async def _init_broker(self):
        topics_set = set()

        for device in self.home.devices:
            topics_set.add(f"message/{device.deviceId}")

        self.broker.set_topics(list(topics_set))
        loop = asyncio.get_event_loop()
        # Listen for mqtt messages in an (unawaited) asyncio task
        task = loop.create_task(self.broker.connect(self.get_credentials))
        # Save a reference to the task so it doesn't get garbage collected
        self.background_tasks.add(task)
        task.add_done_callback(self.background_tasks.remove)

    async def get_credentials(self):
        try:
            await self._authenticate_cognito()
        except:  # noqa: E722
            print("Failed to fetch credentials")
        return self.user.credentials

    async def _authenticate_cognito(self):
        payload = {"IdentityId": constants.identityId}

        headers = {
            "x-amz-target": "AWSCognitoIdentityService.GetCredentialsForIdentity",
            "cache-control": "no-store",
            "x-amz-user-agent": "aws-amplify/5.3.8 framework/201",
            "host": "cognito-identity.ap-south-1.amazonaws.com",
            "connection": "Keep-Alive",
            "user-agent": "okhttp/4.9.2",
            "content-type": "application/x-amz-json-1.1",
        }

        response = await self.http.post(
            constants.cognitoUrl, data=json.dumps(payload), headers=headers
        )

        if response.status != 200:
            raise Exception("Authentication failed")

        data = await response.json(content_type=None)
        credentials = data["Credentials"]
        return credentials

    async def _authenticate(self, mobile: str, password: str):
        credentials = await self._authenticate_cognito()

        payload = {
            "mobile": mobile,
            "password": password,
        }

        response = await self.http.post(constants.loginUrl, json=payload)

        if response.status == 200:
            json = await response.json()
            data = json["data"]
            self.user = User(
                accesstoken=data["accesstoken"],
                email=data["profile"]["email"],
                name=data["profile"]["name"],
                homeId=data["profile"]["selectedHouse"],
                mobile=data["profile"]["mobile"],
                credentials=credentials,
            )
            self.username = mobile
            self.password = password
        else:
            raise Exception("Authentication failed")

    # Get home details
    async def get_home_details(self):
        response = await self.http.get(
            f"{constants.homeDetailsUrl}/{self.user.homeId}/{self.user.mobile}"
        )
        resp = await response.json()
        await self.process_home_details(resp)

    async def process_home_details(self, json_data):
        devices: list[Device] = []

        for room in json_data["rooms"]:
            for device in room["devices"]:
                for switch in device["switches"]:
                    item = Device(
                        deviceId=device["deviceId"],
                        switchId=switch["switchId"],
                        name=switch["switchName"],
                        type=parse_device_type(switch["deviceType"]),
                        switchState=switch["switchState"],
                        dimmable=switch["dimmable"],
                        wattage=switch["wattage"],
                        roomName=room["roomName"],
                        broker=self.broker,
                    )
                    devices.append(item)

        self.home = Home(
            id=json_data["_id"], name=json_data["houseName"], devices=devices
        )



import asyncio
import ssl
import json
import certifi
import aws_signv4_mqtt
from urllib.parse import urlparse
from aiohttp import ClientSession
from aiomqtt import Client, Message, MqttError

from . import constants


def isAsync(someFunc):
    return asyncio.iscoroutinefunction(someFunc)


class AlisteBroker:
    reconnect_interval = 5  # In seconds
    connected = False

    def __init__(self):
        self.http = ClientSession()

        self.callbacks = []

    async def __aenter__(self):
        return self

    async def __aexit__(self, *excinfo):
        await self.disconnect()

    async def connect(self, get_credentials):
        while True:
            credentials = await get_credentials()

            ws_url = aws_signv4_mqtt.generate_signv4_mqtt(
                f"{constants.iotId}.iot.{constants.region}.amazonaws.com",
                constants.region,
                credentials["AccessKeyId"],
                credentials["SecretKey"],
                session_token=credentials["SessionToken"],
            )

            urlparts = urlparse(ws_url)

            # Host header needs to be set, port is not included in signed host header so should not be included here.
            # No idea what it defaults to but whatever that it seems to be wrong.
            headers = {
                "Host": "{0:s}".format(urlparts.netloc),
            }

            self.client = Client(
                hostname=urlparts.netloc,
                port=443,
                transport="websockets",
                websocket_path="{}?{}".format(urlparts.path, urlparts.query),
                websocket_headers=headers,
                tls_context=ssl.create_default_context(cafile=certifi.where()),
            )
            try:
                async with self.client:
                    async with self.client.messages() as messages:
                        await self.on_connect()

                        async for message in messages:
                            self.on_message(message)

            except MqttError as error:
                self.connected = False

                print(
                    f'Error "{error}". Reconnecting in {self.reconnect_interval} seconds.'
                )

                await asyncio.sleep(self.reconnect_interval)

    async def disconnect(self):
        await self.http.close()

    def register_callback(self, callback):
        self.callbacks.append(callback)

    def set_topics(self, topics: list[str]):
        self.commandTopics = topics

    async def on_connect(self):
        self.connected = True

        for topic in self.commandTopics:
            # print("Subscribing to topic: ", topic)
            await self.client.subscribe(topic)

    def on_message(self, message: Message):
        deviceId = message.topic.value.split("/")[1]
        parsed = json.loads(message.payload.decode("utf-8"))
        data = {
            "deviceId": deviceId or 0,
            "switchId": parsed["sid"] or 0,
            "state": (parsed["s"] or 0) / 100,
        }
        for callback in self.callbacks:
            callback(data)

    def message(self, data):
        for callback in self.callbacks:
            callback(data)

    @property
    def is_connected(self):
        return self.connected

    async def send_command(self, payload: dict):
        if self.is_connected:
            deviceId = payload["deviceId"]
            switchId = payload["switchId"]
            command = payload["command"]
            await self.client.publish(
                f"control/{deviceId}", f"{switchId},{command * 100}"
            )
        else:
            await self.http.post(constants.commandUrl, json=payload)
            payload["state"] = payload["command"]
            self.message(payload)



from .device import Device


class Home:
    def __init__(self, id: str, name: str, devices: list[Device]):
        self.id = id
        self.name = name
        self.devices = devices

    def get_device(self, device_id: str):
        for device in self.devices:
            if device.id == device_id:
                return device



baseUrl = "https://web.alistetechnologies.com"
baseUrl2 = "https://a3.alistetechnologies.com"
wssUrl = "https://a2.alistetechnologies.com/?"
loginUrl = "https://web.alistetechnologies.com/v2/auth/login"
homeDetailsUrl = "https://a3.alistetechnologies.com/api/fetch/house2"
commandUrl = "https://a3.alistetechnologies.com/v3/device/control"
region = "ap-south-1"
cognitoUrl = f"https://cognito-identity.{region}.amazonaws.com"
identityId = f"{region}:4b454fb8-df98-4b94-a5ba-937169c43ad0"
iotId = "a1pv71f2h6guqz-ats"



class User:
    accesstoken: str
    email: str
    name: str
    homeId: str
    mobile: str
    credentials: dict

    def __init__(self, accesstoken, email, name, homeId, mobile, credentials):
        self.accesstoken = accesstoken or ""
        self.email = email or ""
        self.name = name or ""
        self.homeId = homeId or ""
        self.mobile = mobile or ""
        self.credentials = credentials or {}



from .broker import AlisteBroker
from .enums import DeviceType


class Device:
    def __init__(
        self,
        deviceId: str,
        switchId: int,
        name: str,
        type: DeviceType,
        switchState: float,
        dimmable: bool,
        wattage: int,
        roomName: str,
        broker: AlisteBroker,
    ):
        self.deviceId = deviceId
        self.switchId = switchId
        self.name = name
        self.type = type
        self.switchState = switchState
        self.dimmable = dimmable
        self.wattage = wattage
        self.roomName = roomName
        self.broker = broker

        self._callbacks = set()
        self.broker.register_callback(self.on_message)

    def on_message(self, message):
        if (
            message["deviceId"] == self.deviceId
            and message["switchId"] == self.switchId
        ):
            self.on_state_change(message["state"])

    def refresh(self):
        for callback in self._callbacks:
            callback()

    def register_callback(self, callback) -> None:
        """Register callback, called when Roller changes state."""
        self._callbacks.add(callback)

    def remove_callback(self, callback) -> None:
        """Remove previously registered callback."""
        self._callbacks.discard(callback)

    def on_state_change(self, state):
        self.switchState = float(state)
        self.refresh()

    def build_command(self, command: float):
        return {
            "deviceId": self.deviceId,
            "switchId": self.switchId,
            "command": command,
        }

    async def turn_on(self):
        await self.broker.send_command(self.build_command(1))

    async def turn_off(self):
        await self.broker.send_command(self.build_command(0))

    async def dim(self, value: float):
        await self.broker.send_command(self.build_command(value))
